/*
*  source code metrics counter
*  Copyright (C) 2015-2016  NEC Corporation.
*  
*  This program is free software; you can redistribute it and/or
*  modify it under the terms of the GNU General Public License
*  as published by the Free Software Foundation; either version 2
*  of the License, or (at your option) any later version.
*  
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#define _CRT_SECURE_NO_WARNINGS
#define MAX_STR_SIZE 3000

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <tchar.h>
#include <sys/stat.h>
#include <mxml.h>
#include "CDMtrCmn.h"
#include "CDMtrExtCmn.h"
#include "CDMtrLibFileW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrLibSystemW.h"
#include "CDMtrLibOptW.h"
#include "CDMtrCntMain.h"
#include "CDMtrCntLang.h"
#include "CDMtrCntFunc.h"
#include "CDMtrCntCallProc.h"
#include "CDMtrCntPreFormat.h"
#include "CDMtrCntXrefDataUtil.h"
#include "CDMtrCntStructure.h"
#include "CDMtrCntLog.h"
#include "CDMtrCntOutputCSV.h"
#include "CDMtrCntExtTool.h"
#include "CDMtrCntFanIO.h"
#include "CDMtrCntLineCount.h"
#include "CDMtrSps.h"
#include "CDMtrMessage.h"

/*----------------------------------------------------------------------------*/
/* static function declaration                                                */
/*----------------------------------------------------------------------------*/
static int   cdmtrCntStructureDoStruct(_TCHAR *pcFileName);
static int   cdmtrCntStructureCheckLangType(int *piShortPathFlag, int *piCharsetFlg, _DoStruct *ptParam);
static int   cdmtrCntStructureCodeMetricsJAVA();
static int   cdmtrCntStructureCodeMetricsCS(_DoStruct *ptParam);
static void  cdmtrCntStructureCodeMetricsVB(_DoStruct *ptParam);
static void  cdmtrCntStructureCodeMetricsJS(_TCHAR *pcFileName, _DoStruct *ptParam);
static int   cdmtrCntStructureCodeMetricsSimple(_DoStruct *ptParam);
static int   cdmtrCntStructureCodeMetricsStandard(_TCHAR *pcFileName, _DoStruct *ptParam);
static int   cdmtrCntStructureDumpAllListsIntoXML();
static int   cdmtrCntStructureDumpListIntoXML(_DoStructNode *doStructNode, int langType);
static char* cdmtrCntStructureGetXMLFileName(int langType);
static int   cdmtrCntStructureMergeAndSortOutput(_FileList *pcFiles);
static int   cdmtrCntStructureMergeAndSortCSVOutput(_FileList *pFileList, _TCHAR * newNcmeterFile);
static void  cdmtrCntStructureReleaseDoStruct(_DoStruct doStruct);
static void  cdmtrCntStructureReleaseDoStructNode(_DoStructNode *doStructNode);
static int   cdmtrCntStructureAddDoStructNode(_DoStruct doStruct);
static void  cdmtrCntStructureReleaseDoStructLists();
static int   cdmtrCntStructureFetchAndUpdateINumOfSuccessJava();
static int   cdmtrCntStructureInitializeDoStructLists();
static int   cdmtrCntStructureInvokeAndProcess(int langType);
static int   cdmtrCntStructureAppendParserOutput(_TCHAR *resultFile); 
static int   cdmtrCntStructureGetAbsFilePath(char *line, char **charRet);
static int   cdmtrCntStructurePrintErrToOutFile(FILE *currOpen, _TCHAR *currOpenName, _TCHAR *fileInList);
static char* cdmtrCntStructureReadDataFromFile(FILE *fileToRead);

/*----------------------------------------------------------------------------*/
/* global variable declaration                                                */
/*----------------------------------------------------------------------------*/

static _DoStructLists *doStructLists = NULL;
static int mergedOutputPos = SEEK_SET;
static int javaOutputPos = SEEK_SET;
static int otherOutputPos = SEEK_SET;
size_t maxStrSize = MAX_STR_SIZE;

/*----------------------------------------------------------------------------*/
/* global function                                                            */
/*----------------------------------------------------------------------------*/
/*!
@brief      \gNX𑪒肷
*/
int CDMtrCntStructureDoStructProcess(_FileList *pcFileName)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    int i = 0;
    int j = 0;
    int k = 0;
    int iLangType = 0;
    _FileList *pFileList = NULL;                               /** List of files in the given directory. */
    _FileList **pArr = NULL;                                   /** Store the link to all _FileList instances. */ 
    fanCounterList *pFanCountList = NULL;
    FILE *fileToDel = NULL;
    _TCHAR *pcMergeFile = NULL;
    _TCHAR *fanIOFile = NULL;
    const _TCHAR *pcFileNameLogOut = NULL;
    _TCHAR *otherOutput = NULL;
    _TCHAR *mergedOutputFile = NULL;

    if( pcFileName == NULL ){
        goto FUNC_END;
    }
    /** Initialize the data type that holds all the _DoStruct instances. */
    iFuncRet = cdmtrCntStructureInitializeDoStructLists();
    if( iFuncRet != RET_OK ){
        goto FUNC_END;
    }
    if( !(iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) ){
        /** Create the absolute path of the file into which parser data(except java) will be appended. */
        iFuncRet = CDMtrLibUtilCreateStringW(&otherOutput, pcTempFolderName, FOLDER_SEPALATE, TEMP_OTHER_OUTPUT, NULL);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        if ((spcOutFile = _tfopen(otherOutput, _T("a+"))) == NULL) {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000114, otherOutput); //* MSG_LOG_ID_C_00000114
            goto FUNC_END;
        }
    }
    iFuncRet = CDMtrLibUtilMallocW((void **)& (pArr), (pcFileName->count) * sizeof(_FileList *));
    if(iFuncRet != RET_OK){
        goto FUNC_END;
    }
    CDMtrLibUtilFreeW((void **)&pcTempFio);
    iFuncRet = CDMtrLibUtilCreateStringW(&pcTempFio, pcTempFolderName, FOLDER_SEPALATE, TEMP_FILE_FIO, NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    CDMtrLibUtilFreeW((void **)&pcTempXref);
    iFuncRet = CDMtrLibUtilCreateStringW(&pcTempXref, pcTempFolderName, FOLDER_SEPALATE, TEMP_FILE_XREF, NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    CDMtrLibUtilFreeW((void **)&mergedOutputFile);
    iFuncRet = CDMtrLibUtilCreateStringW(&mergedOutputFile, pcTempFolderName, FOLDER_SEPALATE, TEMP_MERGED_OUTPUT, NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    for (j = 0; j < pcFileName->count; j++) {
        iFuncRet = CDMtrLibFileGetFileList(pcFileName->filename[j], &pFileList);
        pArr[j] = pFileList;
        pcFileNameLogOut = CDMtrLibFileRemoveLongPathMarkW(pcFileName->filename[j]);
        if (iFuncRet != RET_OK) {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000148, pcFileNameLogOut); //* MSG_LOG_ID_C_00000148
            goto FUNC_END;
        }
        if (pFileList->count == 0) {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000149, pcFileNameLogOut); //* MSG_LOG_ID_C_00000149
        }
        else {  /** Loop through the 'pFileList' and process the files in it. */
            for (i = 0; i < pFileList->count; i++) {      
                iFuncRet = cdmtrCntStructureDoStruct(pFileList->filename[i]);				
                if (iFuncRet == RET_NG) {
                }                    
            }			
        }		
    }

    if( !(iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) ){
        if(spcOutFile != NULL){
            fclose(spcOutFile);
            spcOutFile = NULL;
        }
    }
    if ( doStructLists != NULL ) {

            /** Write 'doStructLists' data into an xml file which will then be used by various language specific parsers.
            *  Currently information related to JAVA input files only is being dumped into the file.
            */
            iFuncRet = cdmtrCntStructureDumpAllListsIntoXML();
            if (iFuncRet == RET_NG){
                CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000726); 
                goto FUNC_END;
            }
            /** Invoke the parser module and process the output produced. */
            if(doStructLists -> javaList != 0){
                cdmtrCntStructureInvokeAndProcess(LANG_TYPE_JAVA);
            }            
    }
    /** Merge the output from various parser modules. */
    cdmtrCntStructureMergeAndSortOutput(pcFileName);

#ifndef OSS
    if (iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) {
        pFanCountList = (fanCounterList *)calloc(1, sizeof(fanCounterList));
        if (pFanCountList == NULL) {
            goto FUNC_END;
        }
        iFuncRet = CDMtrCntFanIOMain(pcTempXref, pFanCountList);
        if (iFuncRet != RET_OK) {
            pFanCountList->iListSize = 0;
        }
        CDMtrLibUtilFreeW((void **)&pcMergeFile);
        iFuncRet = CDMtrLibUtilCreateStringW(&pcMergeFile, pcTempFolderName, FOLDER_SEPALATE, TEMP_MERGE, NULL);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        /** The following section of code populates data into the file pointed
         *  to by pcMergeFile by processing either pcTempFio or mergedOutputFile.
         *  If java input files were present, then we wil use mergedOutputFile
         *  otherwise we'll use pcTempFio.
         */
        if( doStructLists != NULL && doStructLists -> javaList != 0 ){
            iFuncRet = CDMtrCntXrefMerge(mergedOutputFile, pFanCountList, pcMergeFile);
        } else {
            iFuncRet = CDMtrCntXrefMerge(pcTempFio, pFanCountList, pcMergeFile);
        }
        if (iFuncRet == RET_OK) {
            iLangType = 0;
            iFuncRet = CDMtrCntOutputCSVStructFileToOutFile(pcMergeFile, iLangType, maxStrSize);
        }
        if(( fileToDel = _tfopen(pcTempFio, _T("r"))) != NULL){
            fclose(fileToDel);
            fileToDel = NULL;
            /** Delete File. */
            if( CDMtrLibFileDeleteW(pcTempFio) != RET_OK ){
                CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000582, pcTempFio); //* MSG_LOG_ID_C_00000582
            }
        }
    }
#endif //OSS
    if(( fileToDel = _tfopen(mergedOutputFile, _T("r"))) != NULL){
        fclose(fileToDel);
        fileToDel = NULL;
        /** Delete File. */
        if( CDMtrLibFileDeleteW(mergedOutputFile) != RET_OK ){
            CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000582, mergedOutputFile); //* MSG_LOG_ID_C_00000582
        }
    }

    iRet = RET_OK;
FUNC_END:
    if( pArr != NULL){
        for(k=pcFileName->count-1;k>=0;k--){
            CDMtrLibFileFreeFileList(&pArr[k]);
        }
    }
    CDMtrLibUtilFreeW((void **)&pArr);
    cdmtrCntStructureReleaseDoStructLists();
    CDMtrLibUtilFreeW((void **)&pcTempFio);
    CDMtrLibUtilFreeW((void **)&mergedOutputFile);
    CDMtrLibUtilFreeW((void **)&pcTempXref);
    CDMtrLibUtilFreeW((void **)&pcMergeFile);
    CDMtrLibUtilFreeW((void **)&otherOutput);
    CDMtrCntXrefDataUtilFreeAllXrefTables();
    if (pFanCountList != NULL) {
        free(pFanCountList);
    }
    return iRet;
}

/** Function : cdmtrCntStructureMergeAndSortOutput
 *  ----------------------------------------------------
 *  Merges the output of various parser modules into a 
 *  single file for all instances of _FileList.
 *
 *  @param  pcFiles : List of input files/folders.
 *
 *  @return iRet    : Success or failure.
 */
static int cdmtrCntStructureMergeAndSortOutput(_FileList *pcFiles)
{

    /** Temp variable for _FileList instance. */
    _FileList *pFileList = NULL;
    /** Variable for looping pcFiles. */
    int counterFileList = 0;
    /** Function call return value. */
    int iFuncRet = RET_NG;
    int iRet = RET_NG;
    _TCHAR *mergedOutputFile = NULL;
    _TCHAR *javaOutputFile = NULL; 
    _TCHAR *otherOutputFile = NULL;
    FILE *fileToDel = NULL;

    /** Set the name of the file that will have the merged data from output of java
     *  and non-java parsers.
     */
    CDMtrLibFileSetDirFileName(pcTempFolderName, TEMP_MERGED_OUTPUT, &mergedOutputFile);
    /** Set the name of the file that will have output from the java parser. */
    CDMtrLibFileSetDirFileName(pcTempFolderName, TEMP_JPARSER_OUTFILE, &javaOutputFile);
    /** Set the name of the file that will have output from the non-java parsers. Based
     *  on what the command mode and options are, this pointer would contain names of 
     *  different files. If the user entered 'struct_std' command and set the FANIO
     *  property in the properties file, then the file pointed to by pcTempFio will be
     *  used otherwise 'otherOutput' file will be used.
     */
    if( (iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) ){
        otherOutputFile = pcTempFio;
    }
    else{
        CDMtrLibFileSetDirFileName(pcTempFolderName, TEMP_OTHER_OUTPUT, &otherOutputFile);
    }

    /** Merging of the java output file and the non-java output file into the merged output
     *  file is to be done only if there is an input java file. If not then merging process
     *  is to be avoided.
     */
    if ( doStructLists != NULL && doStructLists -> javaList != 0 ){
        for (counterFileList = 0; counterFileList < pcFiles->count; counterFileList++) {
            iFuncRet = CDMtrLibFileGetFileList(pcFiles->filename[counterFileList], &pFileList);
            if (iFuncRet != RET_OK || pFileList == NULL ) {
                CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000729, __FILE__, __LINE__);
                goto FUNC_END;
            }
            iFuncRet = cdmtrCntStructureMergeAndSortCSVOutput(pFileList, mergedOutputFile);
            if ( iFuncRet != RET_OK ) {
                CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000729, __FILE__, __LINE__);
                if (pFileList != NULL) {
                    CDMtrLibFileFreeFileList(&pFileList);
                    pFileList = NULL;
                }
                goto FUNC_END;
            }
            if (pFileList != NULL) {
                CDMtrLibFileFreeFileList(&pFileList);
                pFileList = NULL;
            }
        }
    }

    if ( !(iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE)){
        /** Open the final output file for writing the data. */
        if ((spcOutFile = _tfopen(pcOutFileName, _T("a") ) ) == NULL){
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000114, pcOutFileName); //* MSG_LOG_ID_C_00000114
            goto FUNC_END;
        }
        /** The following call transfers data into the final output
          * file (ncmeter.csv). If there was no input java file, then the merging
          * process did not take place and the data will be transfered from
          * otherOutput into the final output. If however, there was a java input
          * file then data will be transfered from mergedOutput into the final
          * output file.
          */
        if( doStructLists != NULL && doStructLists -> javaList !=0 ){
            iFuncRet = CDMtrCntOutputCSVStructFileToOutFile(mergedOutputFile, LANG_TYPE_JAVA, maxStrSize);
        } else {
            iFuncRet = CDMtrCntOutputCSVStructFileToOutFile(otherOutputFile, LANG_TYPE_JAVA, maxStrSize);
        }
        if(iFuncRet != RET_OK){
            return RET_NG;
        }
    }
    iRet = RET_OK;

FUNC_END:
    javaOutputPos = SEEK_SET;
    mergedOutputPos = SEEK_SET;
    otherOutputPos = SEEK_SET;
    /** If there is no input java file then otherOutputFile points
      * to temp_fanio file. This file is to be used later on for copying the
      * data into the final output file (ncmeter.csv). Hence this file is not
      * to be deleted.
      */
    if ( !(iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) ){
        if(( fileToDel = _tfopen(otherOutputFile, _T("r"))) != NULL){
            fclose(fileToDel);
            fileToDel = NULL;
            /** Delete File. */
            if( CDMtrLibFileDeleteW(otherOutputFile) != RET_OK ){
                CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000582, otherOutputFile); //* MSG_LOG_ID_C_00000582
            }
        }
    }
    if(( fileToDel = _tfopen(javaOutputFile, _T("r"))) != NULL){
        fclose(fileToDel);
        fileToDel = NULL;
        /** Delete File. */
        if( CDMtrLibFileDeleteW(javaOutputFile) != RET_OK ){
            CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000582, javaOutputFile); //* MSG_LOG_ID_C_00000582
        }
    }
    CDMtrLibUtilFreeW((void **)&mergedOutputFile);
    if ( !(iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE)){
        CDMtrLibUtilFreeW((void **)&otherOutputFile);
    }
    CDMtrLibUtilFreeW((void **)&javaOutputFile);
    if (pFileList != NULL) {
        CDMtrLibFileFreeFileList( &pFileList );
        pFileList = NULL;
    }
    return iRet;
}

/** Function : cdmtrCntStructureMergeAndSortCSVOutput
 *  ----------------------------------------------------
 *  Merges the output of java and non-java parsers 
 *  in a sorted manner into a single file for a 
 *  single instance of _FileList.
 *
 *  @param  fileList         : List of input file
 *                             (to be used for sorting).
 *
 *  @param  mergedOutputFile : File into which merged
 *                             data will be dumped.
 *
 *  @return iRet             : Success or failure.
 */
static int cdmtrCntStructureMergeAndSortCSVOutput(_FileList *fileList, _TCHAR * mergedOutputFile)
{

    int counterFileList = 0;
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    char *lastJavaStr = NULL;
    char *lastOtherStr = NULL;
    char *fileAbsPath = NULL;
    const _TCHAR *inputFileAbsPath = NULL;
    _TCHAR *javaOutputFile = NULL;
    _TCHAR *otherOutputFile = NULL;
    static FILE *mergedOutput = NULL;
    static FILE *javaOutput = NULL;
    static FILE *otherOutput = NULL;
    int iLangType = 0;
    int maxStrLength = MAX_STR_SIZE;
    _TCHAR * pcLangName = NULL;
    boolean atLeastOne = FALSE;
    char *mbcsFileName = NULL;
    boolean noJavaOutput = FALSE;
    
    struct _stat buf ;


    /** Assign the name of the java output file to javaOutputFile. */
    CDMtrLibFileSetDirFileName(pcTempFolderName, TEMP_JPARSER_OUTFILE, &javaOutputFile);
    if((javaOutput = CDMtrLibUtilOpenAndSetFilePointer( javaOutputFile, _T("r"), javaOutputPos )) == NULL){
        /** Set a flag to indicate no java file present. */
        noJavaOutput = TRUE;
    }
    /** Set the name of the output file of non-java parsers. */
    if( (iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) ){
        otherOutputFile = pcTempFio;
    }else{
        CDMtrLibFileSetDirFileName(pcTempFolderName, TEMP_OTHER_OUTPUT, &otherOutputFile);
    }
    /**Open the file for reading and set the file pointer accordingly. */
    if(!( _wstat (otherOutputFile, &buf) != 0 && errno == ENOENT ))
    {
          if(( otherOutput = CDMtrLibUtilOpenAndSetFilePointer( otherOutputFile, _T("r"), otherOutputPos )) == NULL){
               goto FUNC_END;
          }
    }

    /**Loop through pFileList to get the names of all the input files. */
    for (counterFileList = 0; counterFileList < fileList->count; counterFileList++) {

        /** Open/create file containing the sorted data in appending mode. */
        if((mergedOutput = CDMtrLibUtilOpenAndSetFilePointer( mergedOutputFile, _T("a"), mergedOutputPos )) == NULL){
            goto FUNC_END;
        }

        atLeastOne = FALSE;
        mbcsFileName = NULL;

        /** Convert the file path from unicode to mcbs. */		         
        inputFileAbsPath = CDMtrLibFileRemoveLongPathMarkW(fileList->filename[counterFileList]); 
        iRet = CDMtrLibUtilUNICODEtoMBCS(&mbcsFileName, &inputFileAbsPath, ENCODING_UTF_8);
        if (iRet != RET_OK) {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000729); //* MSG_LOG_ID_C_00000729
            continue;
        }

        /** First a check to confirm whether the file in 'fileList' is indeed 
        *  of the 6 expected types and not something else(like .h,.txt etc). 
        */
        iLangType = CDMtrCntLangGetTypeFromFileName(giExceptBinaryMode, fileList->filename[counterFileList] , &pcLangName);

        if ( CDMtrLibUtilIsValidExtension ( iLangType ) == RET_OK ) {

            if ( iLangType == LANG_TYPE_JAVA ) {                            /** If input file is in java. */
                if(noJavaOutput){
                    cdmtrCntStructurePrintErrToOutFile(mergedOutput, mergedOutputFile, fileList->filename[counterFileList]);
                }else{
                    while( 1 ) {
                        /** Get the current file pointer position. */
                        javaOutputPos = ftell(javaOutput);
                        /** Read data from the file. */
                        if(( lastJavaStr = cdmtrCntStructureReadDataFromFile(javaOutput)) == NULL )
                             break;
                        /** Need to check if line read has file name and is matching in word and case.
                         *  Extract the exact file name from the line read for this purpose.
                         */
                        if((cdmtrCntStructureGetAbsFilePath( lastJavaStr, &fileAbsPath )) != RET_OK)
                            break;

                        /** Check if line needs to be written and take appropriate action. */
                        if((strcmp( fileAbsPath, mbcsFileName ) == 0)){
                            /** Line read corresponds to the filename, write it in newNcmeter. */
                            fputs( lastJavaStr, mergedOutput);
                            atLeastOne = TRUE;
                            CDMtrLibUtilFreeW((void **)&fileAbsPath);
                            CDMtrLibUtilFreeW((void **)&lastJavaStr);
                        }
                        /** If no data exists for the given file. */
                        else if(!atLeastOne){
                            /** Roll back the file pointer of the java output. */
                            fseek( javaOutput, javaOutputPos, SEEK_SET);
                            //call
                            cdmtrCntStructurePrintErrToOutFile(mergedOutput, mergedOutputFile, fileList->filename[counterFileList]);
                            mergedOutputPos = ftell(mergedOutput);
                            break;
                        }
                        else
                        {   /** Roll back the javaOutput file pointer. */
                            fseek( javaOutput, javaOutputPos, SEEK_SET );
                            break;
                        }
                    }
                    CDMtrLibUtilFreeW((void **)&fileAbsPath);
                    CDMtrLibUtilFreeW((void **)&lastJavaStr);
                }
            }
            else{				
                while(1){

                    /** Get the current file pointer position. */
                    otherOutputPos = ftell(otherOutput);
                    /** Read data from file. */
                    if( (lastOtherStr = cdmtrCntStructureReadDataFromFile(otherOutput)) == NULL )
                        break;
                    /** Put a check on fgets and take action. */

                    /** Need to check if line read has file name and is matching in word and case.
                    * Extract the exact file name from the line read for this purpose.
                    */
                    if((cdmtrCntStructureGetAbsFilePath( lastOtherStr, &fileAbsPath )) != RET_OK)
                        break;

                    /** Check if line needs to be written and take appropriate action. */
                    if((strcmp( fileAbsPath, mbcsFileName ) == 0)){
                        /** Line read corresponds to the filename, write it in newNcmeter. */
                        fputs( lastOtherStr, mergedOutput );
                        CDMtrLibUtilFreeW((void **)&fileAbsPath);
                        CDMtrLibUtilFreeW((void **)&lastOtherStr);
                    }
                    else
                    {   /** Roll back the javaOutput file pointer. */
                        fseek( otherOutput, otherOutputPos, SEEK_SET );
                        break;
                    }                    
                }
                CDMtrLibUtilFreeW((void **)&fileAbsPath);
                CDMtrLibUtilFreeW((void **)&lastOtherStr);
            }
        }
        /** 'newNcmeter' needs to be closed repeatedly to avoid overflowing the memory alloted. */
        mergedOutputPos = CDMtrLibUtilCloseAndStoreFilePointer(mergedOutput);
        CDMtrLibUtilFreeW((void **)&pcLangName);
        CDMtrLibUtilFreeW((void **)&mbcsFileName);
    }
    iRet = RET_OK;
FUNC_END:
    /**Release memory held up by various variable. */
    CDMtrLibUtilFreeW((void **)&javaOutputFile);
    if( !(iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) ){
        CDMtrLibUtilFreeW((void **)&otherOutputFile);
    }
    CDMtrLibUtilFreeW((void **)&fileAbsPath);

    /**Close all the files. */
    if( otherOutput != NULL ){
        otherOutputPos = CDMtrLibUtilCloseAndStoreFilePointer(otherOutput);
    }
    if( javaOutput != NULL ){
        javaOutputPos = CDMtrLibUtilCloseAndStoreFilePointer(javaOutput);
    }
    return iRet;
}

/*!
@brief          \gNX̑茋ʂ̊mFNESTS␳s
*/
int CDMtrCntStructureStructRecordCheckAndMod(_TCHAR *pcBuffer)
{
    int iRet = RET_NG;
    int iItem = STRUCT_ITEM_NUM_FIFO;
    int iLen = 0;
    int iLenNew = 0;
    int iNests = 0;
    _TCHAR *p = NULL;
    _TCHAR *pcNestsPos = NULL;
    _TCHAR cNestsStr[STRUCT_NESTS_STR_LEN];
    if (pcBuffer == NULL) {
        return iRet;
    }
    p = &(pcBuffer[_tcslen(pcBuffer) - 1]);
    for (; p != pcBuffer; p--) {
        if (*p != _T(',')) {
            continue;
        }
        iItem--;
        if (iItem <= STRUCT_ITEM_DIR) {
            break;
        }
        else if (iItem == STRUCT_ITEM_NESTS) {
            pcNestsPos = p;
        }
        else if (iItem == (STRUCT_ITEM_NESTS - 1)) {
            iLen = pcNestsPos - p - 1;
            if ((iNestsCount != NUM_NESTS_COUNT_INNERBLOCK) || (iLen <= 0) || (iLen >= STRUCT_NESTS_STR_LEN)) {
                continue;
            }
            _tcsncpy(cNestsStr, p + 1, iLen);
            cNestsStr[iLen] = _T('\0');
            errno = 0;
            iNests = _ttoi(cNestsStr);
            if ((errno != 0) || (iNests <= 0)) {
                continue;
            }
            iNests--;
            memset(cNestsStr, 0x00, STRUCT_NESTS_STR_LEN);
            if (_itot_s(iNests, cNestsStr, STRUCT_NESTS_STR_LEN, 10) != 0) {
                goto FUNC_END;
            }
            iLenNew = _tcslen(cNestsStr);
            _tcsncpy((p + 1), cNestsStr, iLenNew);
            if (iLen != iLenNew) {
                _tcscpy((p + 1 + iLenNew), pcNestsPos);
            }
        }
    }
    if (iItem <= STRUCT_ITEM_DIR) {
        iRet = RET_OK;
    }
    else {
        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000168); //* MSG_LOG_ID_C_00000168
    }
FUNC_END:
    return iRet;
}

/*----------------------------------------------------------------------------*/
/* static function                                                            */
/*----------------------------------------------------------------------------*/
/*!
@brief      \gNX𑪒肷(Pt@C̑)
*/

/** Function : cdmtrCntStructureDoStruct
 *  ---------------------------------------
 *  Populates a _DoStruct instance with info
 *  about the file(argument) and invokes the 
 *  corresponding parser module.
 *
 *  @param  pcFileName : Input file for which
 *                       parser module will be
 *                       invoked.
 *
 *  @return iRet       : Success or failure.
 */
static int cdmtrCntStructureDoStruct(_TCHAR *pcFileName)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    int iShortPathFlag = CDMTR_TRUE;
    unsigned int uiLenName = 0;
    int iCharsetFlg = CDMTR_FALSE;
    int iOldNumOfSuccess = iNumOfSuccess;
    _DoStruct tParam;

    memset(&tParam, 0, sizeof(_DoStruct));
    /** Remove "\\?\" prefix from the path of input file and populate tParam. */
    tParam.pcFileNameLogOut = CDMtrLibFileRemoveLongPathMarkW(pcFileName);       
    giStmtMode = CDMTR_FALSE;

    /** Set default debug options. */
    tParam.pcOptionDebug = _T(" ");
    tParam.pcOptionDebug2 = _T(" ");

    /** Oo͗pt@CpXݒ肵ďo͂s */
    /** Set spcTargetFilePath to the path of input files(pcFileNameLogOut). */
    CDMtrLibCmnSetTargetName(tParam.pcFileNameLogOut);
    iSourceID++;
    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000253); //* MSG_LOG_ID_C_00000253
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000254, tParam.pcFileNameLogOut); //* MSG_LOG_ID_C_00000254
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000255, iSourceID); //* MSG_LOG_ID_C_00000255
    }

    /** t@C擾 */
    /** Allocate memory for tParam.pcFName. */
    uiLenName = _tcslen(pcFileName);
    iFuncRet = CDMtrLibUtilMallocW((void **)& (tParam.pcFName), (uiLenName + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** Assign file name to pcFName field. */
    iFuncRet = CDMtrLibFileGetFileName(pcFileName, tParam.pcFName);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** ʂ擾Ďʖ̏s */
    /** Assign int corresponding to the language to tParam.iLangType. */
    tParam.iLangType = CDMtrCntLangGetTypeFromFileName(giExceptBinaryMode, pcFileName, &(tParam.pcLangName));
    globaliLangType = tParam.iLangType;
    iFuncRet = cdmtrCntStructureCheckLangType(&iShortPathFlag, &iCharsetFlg, &tParam);
    if (iFuncRet != RET_OK) {
        if (iFuncRet == RET_SKIP) {
            iRet = RET_OK;
        }
        goto FUNC_END;
    }

    /** Print execution information if 'debug' option is on. */
    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000260, tParam.pcFName); //* MSG_LOG_ID_C_00000260
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000261, tParam.pcLangName); //* MSG_LOG_ID_C_00000261
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000262, tParam.iLangType); //* MSG_LOG_ID_C_00000262
    }

    /** WG[o͗pt@CpX쐬 */
    /** Assign absolute path of the 'temp_error_output.tmp' file to tParam.pcErrOutFile. */
    iFuncRet = CDMtrLibUtilCreateStringW(&(tParam.pcErrOutFile), pcTempFolderName, FOLDER_SEPALATE, TEMP_FILE_ERROUTPUT, NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** Allocate memory for tParam.pcDirPath. */
    iFuncRet = CDMtrLibUtilMallocW((void **)& (tParam.pcDirPath), (_tcslen(tParam.pcFileNameLogOut) + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** Assign path of directory having input file to tParam.pcDirPath. */
    iFuncRet = CDMtrLibFileGetDirName(tParam.pcFileNameLogOut, tParam.pcDirPath);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** ς݃t@CJEg */
    /** Increment the number of files succesfully processed. */
    iNumOfFiles++;
    if (iShowDetails == DETAILS_SHOW) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000263, tParam.pcFileNameLogOut); //* MSG_LOG_ID_C_00000263
    }
    
    /** NKFs */
    /** Create tempnkf file(invoke nkf.dll methods) and set tParam.pcNKfFile to its location. */
    if (tParam.iLangType != LANG_TYPE_JAVA){
        iFuncRet = CDMtrCntExtToolExecNkf(pcFileName, DIFF_NONE, tParam.iLangType, iShortPathFlag, iCharsetFlg, &(tParam.pcNkfFile), tParam.pcErrOutFile);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        /* NKF̃t@C\ */
        /** Print execution information if 'debug' option is on. */
        if (iDebug == DEBUG_ON) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000264, tParam.pcNkfFile); //* MSG_LOG_ID_C_00000264
        }
    }

    /** \[X`Kvȏꍇ͎{ */
    /** Invoke uncrustify.exe input file(except for java file). */
    if (CDMtrCntExtToolIsExecBeautifier(tParam.iLangType) && tParam.iLangType != LANG_TYPE_JAVA) {
        iFuncRet = CDMtrCntExtToolExecBeautifier(DIFF_NONE, tParam.pcLangName, &(tParam.pcNkfFile), tParam.pcErrOutFile);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }



    /** CJE^p̃e|t@C쐬 */
    /** Assign absolute path of the file to dump output (temp5). */
    CDMtrLibUtilFreeW((void **)& (tParam.pcResultFile));
    iFuncRet = CDMtrLibUtilCreateStringW(&(tParam.pcResultFile), pcTempFolderName, FOLDER_SEPALATE, TEMP_FILE5, NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** Following code sections takes actions based on the extension of the input file. */
    if (tParam.iLangType == LANG_TYPE_JAVA) {               
        /** JAVAΉR[hgNXs */
        tParam.pcFileNameLogOut = pcFileName;
        /** Add _DoStruct instance into the javaList. */
        iFuncRet = cdmtrCntStructureAddDoStructNode(tParam);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    else if (tParam.iLangType == LANG_TYPE_CS) {
        /** C#ΉR[hgNXs */
        /** C# Parser Invocation*/
        iFuncRet = cdmtrCntStructureCodeMetricsCS(&tParam);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    else if (tParam.iLangType == LANG_TYPE_VB) {
        /** VBΉR[hgNXs */
        /** Add _DoStruct instance into the vb_ist. */
        cdmtrCntStructureCodeMetricsVB(&tParam);		
    }else if (tParam.iLangType == LANG_TYPE_JS) {
        /** JavaScriptΉR[hgNXs */
        /** JS Parser Invocation*/
        cdmtrCntStructureCodeMetricsJS(pcFileName, &tParam);

    }
    else {
        if (iStructMode == OPTION_STRUCT_SMP) {
            /** C/C++ΉR[hgNXs(ȈՃp[T) */
            /** C/C++ Parser Invocation*/
            iFuncRet = cdmtrCntStructureCodeMetricsSimple(&tParam);
#ifndef OSS
        }
        else if (iStructMode == OPTION_STRUCT_STD) {
            /** C/C++ΉR[hgNXs(Wp[T) */
            iFuncRet = cdmtrCntStructureCodeMetricsStandard(pcFileName, &tParam);
#endif //OSS
        }
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    if ( iNumOfSuccess > iOldNumOfSuccess ){
#ifndef OSS
        if (iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) {
            /** FANIOΉ */
            iFuncRet = CDMtrCntFanIOStructFileToFanTempFile(tParam.pcResultFile, tParam.iLangType, CDMTR_TRUE);
            if (iFuncRet == RET_NG) {
                goto FUNC_END;
            }
        }
        else {
#endif //OSS
            /** Append the data from the parser output into a temp file. */
            iFuncRet = cdmtrCntStructureAppendParserOutput(tParam.pcResultFile);
            if (iFuncRet == RET_NG) {
                goto FUNC_END;
            }
#ifndef OSS
        }
#endif //OSS
        iRet = RET_OK;
    }
FUNC_END:
    if (iRet != RET_OK) {
        /** File is not processed on the parser module end. Print the output specific to
         *  this case in the output file.
         */
        if (tParam.iLangType != LANG_TYPE_JAVA){
#ifndef OSS
            if (iStructMode == OPTION_STRUCT_STD && giFANIOMode == CDMTR_TRUE) {
                CDMtrCntFanIOErrFileToFanTempFile(tParam.pcFileNameLogOut, tParam.pcLangName, TRUE);
            }
            else {
#endif //OSS
                           
                CDMtrCntOutputCSVErrFileToOutFile( tParam.pcFileNameLogOut, tParam.pcLangName, TRUE );           
#ifndef OSS
           }
#endif //OSS
        }
    }
    /** Free up memory held by the '_DoStruct' instance except for java files.
     *  Freeing up of '_DoStruct' instances of java files is performed later.
     */
    if (tParam.iLangType != LANG_TYPE_JAVA ){
        CDMtrLibCmnClearTargetName();
        cdmtrCntStructureReleaseDoStruct( tParam );
    }
    return iRet;
}

/** Function : cdmtrCntStructureAddDoStructNode
 *  -------------------------------------------
 *  Adds the argument into corresponding field
 *  of _DoStructList instance.
 *
 *  @param  DoStruct   : Instance to be added.
 *
 *  @return iRet       : Success or failure.
 */
static int cdmtrCntStructureAddDoStructNode(_DoStruct DoStruct) {

    /** Allocate the _DoStruct argument in an instance of _DoStructNode. */
    _DoStructNode *currDoStructNode = (_DoStructNode *)CDMtrLibCmnMalloc(sizeof(_DoStructNode));
    if (currDoStructNode == NULL){
        CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000731, __FILE__, __LINE__ );
        return RET_NG;
    }
    /** Assign DoStruct to tempDoStructNode. */
    currDoStructNode->doStruct = DoStruct;
    currDoStructNode->next = NULL;		

    switch (DoStruct.iLangType){

    case LANG_TYPE_JAVA:                              /** Add in java list. */
        {
            /** Check if the javalist is null then alloc the memory and 
            * point the head and tail to the current node.
            */
            if ( doStructLists->javaList == NULL ) {
                if ((doStructLists->javaList = (_DoStructList *)CDMtrLibCmnMalloc( sizeof(_DoStructLists))) == NULL ){
                    CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000732, __FILE__, __LINE__ );
                    return RET_NG;
                }
                doStructLists->javaList->listHead = currDoStructNode;
                doStructLists->javaList->listTail = currDoStructNode;
            }
            /** Else point the tail to the current node after making
            *  current node as next to tail node.
            */
            else {
                doStructLists->javaList->listTail -> next = currDoStructNode;
                doStructLists->javaList->listTail = currDoStructNode;
            }
            break;
        }
    case LANG_TYPE_CS:                                /** Add in cs list. */
        break;

    case LANG_TYPE_JS:                                /** Add in js list. */
        break;

    case LANG_TYPE_VB:                                /** Add in vb list. */
        break;

    case LANG_TYPE_C:                                 /** Add in c/cpp list. */
    case LANG_TYPE_CPP:
        break;

    default:                                          /** Default case, do nothing. */
        return RET_NG;

    }

    return RET_OK; 
}

/*!
@brief          \gNX̌ʂ𔻒f
*/
static int cdmtrCntStructureCheckLangType(int *piShortPathFlag, int *piCharsetFlg, _DoStruct *ptParam)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;

    switch (ptParam->iLangType) {
    case LANG_TYPE_CPP:
        /** CPP[hŎs */
        ptParam->pcLangOpt = EDG_OPT_CPP;
        /** --debug2IvVǉB */
        if (iSpsDebug == DEBUG_ON) {
            ptParam->pcOptionDebug2 = STRUCT_OPTION_DEBUG2;
        }
        break;
    case LANG_TYPE_C:
        /** C[hŎs */
        ptParam->pcLangOpt = EDG_OPT_C;
        /** --debug2IvVǉB */
        if (iSpsDebug == DEBUG_ON) {
            ptParam->pcOptionDebug2 = STRUCT_OPTION_DEBUG2;
        }
        break;
    case LANG_TYPE_CS:
        *piShortPathFlag = CDMTR_FALSE;
    case LANG_TYPE_JAVA:
    case LANG_TYPE_JS:
#if NEED_SOURCE_ID
        _stprintf(chSourceID, SOURCEID_FMT, iSourceID);
#endif
        /** charset w̗LmF */
        if (pcCharSet != NULL) {
            /** R[h̃IvVw肷 */
            iFuncRet = CDMtrLibUtilCreateStringW(&(ptParam->pcOptionCharset), _T(" --charset \""),
                pcCharSet, _T("\" "), NULL);
            if (iFuncRet != RET_OK) {
                goto FUNC_END;
            }
            *piCharsetFlg = CDMTR_TRUE;
        }
        else {
            /** ftHgCHARSETݒ肷 */
            iFuncRet = CDMtrLibUtilCreateStringW(&(ptParam->pcOptionCharset), _T(" --charset \""),
                STRUCT_DEFAULT_CHARSET, _T("\" "), NULL);
            if (iFuncRet != RET_OK) {
                goto FUNC_END;	
            }
        }

        /** --debugIvVǉB */
        if (iDebug == DEBUG_ON) {
            ptParam->pcOptionDebug = STRUCT_OPTION_DEBUG;
        }

        /** --debug2IvVǉB */
        if (iSpsDebug == DEBUG_ON) {
            ptParam->pcOptionDebug2 = STRUCT_OPTION_DEBUG2;
        }
        break;
    case LANG_TYPE_VB:
#if NEED_SOURCE_ID
        _stprintf(chSourceID, SOURCEID_FMT, iSourceID);
#endif
        /** --debugIvVǉB */
        if (iDebug == DEBUG_ON) {
            ptParam->pcOptionDebug = STRUCT_OPTION_DEBUG;
        }

        /** --debug2IvVǉB */
        if (iSpsDebug == DEBUG_ON) {
            ptParam->pcOptionDebug2 = STRUCT_OPTION_DEBUG2;
        }
        break;
    case LANG_TYPE_CPPH:
    case LANG_TYPE_CH:
    case LANG_TYPE_TXT:
    case LANG_TYPE_ETC:
        /** XLbv */
        if (iShowDetails == DETAILS_SHOW) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000257, ptParam->pcFileNameLogOut); //* MSG_LOG_ID_C_00000257
        }
        iRet = RET_SKIP;
        goto FUNC_END;
        break;
    case LANG_TYPE_BIN:
        /** XLbv */
        if (iShowDetails == DETAILS_SHOW) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000258, ptParam->pcFileNameLogOut); //* MSG_LOG_ID_C_00000258
        }
        iRet = RET_SKIP;
        goto FUNC_END;
        break;
    case LANG_TYPE_UNKNOW:
    default:
        /** XLbv */
        if (iShowDetails == DETAILS_SHOW) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000259, ptParam->pcFileNameLogOut); //* MSG_LOG_ID_C_00000259
        }
        iRet = RET_SKIP;
        goto FUNC_END;
        break;
    }
    iRet = RET_OK;

FUNC_END:
    return iRet;
}

/*!
@brief          \gNX𑪒肷(JAVA)
*/
static int cdmtrCntStructureCodeMetricsJAVA()
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    _TCHAR *pcCommand = NULL;
    /** Name of the error output file(temp_err_output.tmp). */
    _TCHAR *errorOutput = NULL;
    /** Flag to denote whether to use nkf exe or dll. */
    _TCHAR *nkfExeOrDllFlag = NULL;
    /** Path of either nkf exe or dll. */
    _TCHAR *nkfPath = NULL;
    /** Alias of nkfPath, to be used for de-allocating memory. */
    _TCHAR *nkfPathAlias = NULL;
    /**Path of folder containing nkf dll file. */
    _TCHAR *nkfDllPath = NULL;
    /** Variables used for passing off debug information to the java parser. */
    _TCHAR *debug = _T(" ");                                
    _TCHAR *debug2 = _T(" ");
    _TCHAR *nkfOption = NULL;

    if (iDebug == DEBUG_ON) {
        /** Set debug option explicitly. */
        debug = STRUCT_OPTION_DEBUG;
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000265); //* MSG_LOG_ID_C_00000265
    }
    if (iSpsDebug == DEBUG_ON){
        /** Set debug2 option explicitly. */
        debug2 = STRUCT_OPTION_DEBUG2;
    }

    /** Check if exe or dll needs to be used. */
    if( giProfileNKFEXESetting == CDMTR_TRUE ){ 
        /** Use exe. */
        /** Set Flag to denote that nkf exe will be used. */
        nkfExeOrDllFlag = CDMTR_EXE;
        /** At this point, pcNKfExe contains the path and the 
            option of nkf tool.This value can be in two forms.
            It can be wrapped in double quotes or without it.
          */        
        nkfPath = CDMtrLibUtilCopyStringW(pcNkfExe);
        nkfPathAlias = nkfPath;
        /** If the path/option is wrapped in double quotes, below
            line of code will store the path of nkf.exe into nkfPath,
            else it will store the path and the option into nkfPath.
          */
        if( (nkfPath = wcstok( nkfPath, DOUBLEQUOTATION )) == NULL )
           goto FUNC_END;
        /** If nkfPath contains only the path of nkf.exe, below line 
            of code will store the nkf option into nkfOption, otherwise
            it'll return null in which case a blank space will be stored.
          */
        if( (nkfOption = wcstok( NULL, DOUBLEQUOTATION )) == NULL ){
            nkfOption = OPTION_SEP_SPACE;
        }
    }
    else
    {
        /** Use dll. */
        /** Create string for path of the folder containing nkf dll. */
        iFuncRet = CDMtrLibUtilCreateStringW(&nkfDllPath, pcObjectPath, NKF_DLL_PATH, NULL);
        if( iFuncRet != RET_OK )
            goto FUNC_END;
        /** Set Flag to denote that nkf exe will be used. */
        nkfExeOrDllFlag = CDMTR_DLL;
        /** Set the path of nkf exe. */
        nkfPath = nkfDllPath ;
        nkfOption = _T(" ");
    }
    /** Free pcCommand before allocating. */
    CDMtrLibUtilFreeW((void **)&pcCommand);


    /**Prepare 'pcCommand' to be sent to the java parser. */
    iFuncRet = CDMtrLibUtilCreateStringW(&pcCommand,
        pcStructJAVAPath,                                   /** Command that is used to invoke a java process(java -Xss1024k -Dfile.encoding=utf-8 -jar + path of ncmjparser.jar file ). */
        debug, debug2,                                      /** Debug options. */ 
        _T(" --nkf_prop "), pcNkf,                          /** Flag to denote whether to execute nkf functionality or not. */
        _T(" --nkf_ExeOrDll "), nkfExeOrDllFlag,            /** Flag to denote whether to use nkf exe or dll. */
        _T(" --nkf_path "), DOUBLEQUOTATION,                /** Path of the nkf tool, exe or dll. */
        nkfPath, nkfOption, DOUBLEQUOTATION,
        _T(" --comp_opt "), gpcCompTypeForJava,             /** Version of java being called. */
        _T(" --func_opt "), pcFuncOpt,                      /** Sample value "param". */
        _T(" --output_dir "), pcTempFolderName,             /** Path of the temp folder. */
        _T(" --error_output \""), TEMP_FILE_ERROUTPUT,      /** Name of the error ouput file. */
        NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** Get the absolute path of the 'temp_err_output.tmp' file. */
    iFuncRet = CDMtrLibFileSetDirFileName(pcTempFolderName, TEMP_FILE_ERROUTPUT, &errorOutput);

    /** 쐬R}hs */
    /** Java parser is invoked from within this function call. */
    iFuncRet = CDMtrCntCallCreateProcess(STRUCT_JAVA, pcCommand, NULL, NULL, NULL);
    if (iDebug == DEBUG_ON || iSpsDebug == DEBUG_ON) {
        CDMtrCntLogLogOutErrExec(errorOutput);
    }

    if (iFuncRet != RET_OK) {
        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000151, pcStructJAVAPath); //* MSG_LOG_ID_C_00000151
        iFuncRet = CDMtrCntCallJavaVersion(pcJAVAExeVersion, errorOutput);
        if (iFuncRet != RET_OK) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000266); //* MSG_LOG_ID_C_00000266
        }
        else {
            CDMtrCntMainShowFile(errorOutput);
        }
    }
    iRet = RET_OK;
FUNC_END:
    CDMtrLibUtilFreeW((void **)&pcCommand);
    CDMtrLibUtilFreeW((void **)&errorOutput);
    CDMtrLibUtilFreeW((void **)&nkfPathAlias);	
    return iRet;
}

/*!
@brief          \gNX𑪒肷(C#)
*/
static int cdmtrCntStructureCodeMetricsCS(_DoStruct *ptParam)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    _TCHAR *pcCommand = NULL;

    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000267); //* MSG_LOG_ID_C_00000267
    }
    CDMtrLibUtilFreeW((void **)&pcCommand);

    /** R}h쐬 */
    iFuncRet = CDMtrLibUtilCreateStringW(&pcCommand,
        pcStructCSPath,
        ptParam->pcOptionDebug, ptParam->pcOptionDebug2,
        _T(" --error_output \""), ptParam->pcErrOutFile,
        _T("\" --metric_file \""), ptParam->pcResultFile,
        _T("\" --target_file_path \""), ptParam->pcFileNameLogOut,
        _T("\" --skip \""), pcSkip,
        _T("\" --func_opt \""), pcFuncOpt,
        _T("\" "), ptParam->pcOptionCharset, _T(" \""),
        ptParam->pcNkfFile,
        _T("\""),
        NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** 쐬R}hs */
    iFuncRet = CDMtrCntCallCreateProcess(STRUCT_CS, pcCommand, NULL, NULL, NULL);
    if (iDebug == DEBUG_ON || iSpsDebug == DEBUG_ON) {
        CDMtrCntLogLogOutErrExec(ptParam->pcErrOutFile);
    }
    if (iFuncRet != RET_OK) {
        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000152, pcStructCSPath); //* MSG_LOG_ID_C_00000152
    }
    else {
        iNumOfSuccess++;
    }

    iRet = RET_OK;
FUNC_END:
    CDMtrLibUtilFreeW((void **)&pcCommand);
    return iRet;
}

/*!
@brief          \gNX𑪒肷(VisualBasic)
*/
static void cdmtrCntStructureCodeMetricsVB(_DoStruct *ptParam)
{
    int iFuncRet = RET_NG;

    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000268); //* MSG_LOG_ID_C_00000268
    }

    /** \gNXs */
    iFuncRet = CDMtrCntCallVBParser(pcStructVBPath,
        ptParam->pcOptionDebug, ptParam->pcOptionDebug2,
        ptParam->pcErrOutFile,
        pcTempFolderName,
        ptParam->pcNkfFile,
        chSourceID);
    if (iDebug == DEBUG_ON || iSpsDebug == DEBUG_ON) {
        CDMtrCntLogLogOutErrExec(ptParam->pcErrOutFile);
    }
    if (iFuncRet != RET_OK) {
        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000153, pcStructVBPath); //* MSG_LOG_ID_C_00000153
    }
    else {
        /** Wvs */
        iFuncRet = CDMtrCntPreFormatAggregate(pcTempFolderName,
            chSourceID,
            ptParam->pcDirPath,
            ptParam->pcFName,
            ptParam->pcLangName,
            ptParam->pcResultFile,
            ptParam->iLangType);
        if (iFuncRet != RET_OK) {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000154, ptParam->pcFileNameLogOut); //* MSG_LOG_ID_C_00000154
        }
        else {
            iNumOfSuccess++;
        }
    }
    return;
}

/*!
@brief          \gNX𑪒肷(JavaScript)
*/
static void cdmtrCntStructureCodeMetricsJS(_TCHAR *pcFileName, _DoStruct *ptParam)
{
    int iFuncRet = RET_NG;

    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000269); //* MSG_LOG_ID_C_00000269
    }

    /** \gNXs */
    iFuncRet = CDMtrCntCallJSParser(pcStructJSPath,
        ptParam->pcOptionDebug, ptParam->pcOptionDebug2,
        ptParam->pcErrOutFile,
        pcTempFolderName,
        pcFileName,
        chSourceID,
        ptParam->pcOptionCharset,
        ptParam->pcNkfFile);
    if (iDebug == DEBUG_ON || iSpsDebug == DEBUG_ON) {
        CDMtrCntLogLogOutErrExec(ptParam->pcErrOutFile);
    }
    if (iFuncRet != RET_OK) {
        /** JAVAsVersion\pR}h쐬 */
        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000155, pcStructJSPath); //* MSG_LOG_ID_C_00000155
        iFuncRet = CDMtrCntCallJavaVersion(pcJAVAExeVersion, ptParam->pcErrOutFile);
        if (iFuncRet != RET_OK) {
            CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000270); //* MSG_LOG_ID_C_00000270
        }
        else {
            CDMtrCntMainShowFile(ptParam->pcErrOutFile);
        }
    }
    else {
        /** Wvs */
        iFuncRet = CDMtrCntPreFormatAggregate(pcTempFolderName,
            chSourceID,
            ptParam->pcDirPath,
            ptParam->pcFName,
            ptParam->pcLangName,
            ptParam->pcResultFile,
            ptParam->iLangType);
        if (iFuncRet != RET_OK) {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000156, ptParam->pcFileNameLogOut); //* MSG_LOG_ID_C_00000156
        }
        else {
            iNumOfSuccess++;
        }
    }
}

/*!
@brief          \gNX𑪒肷(C/C++(Ȉ))
*/
static int cdmtrCntStructureCodeMetricsSimple(_DoStruct *ptParam)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    int iCntLangType = 0;
    _TCHAR langTypeBuf[8] = { 0 };
    _TCHAR *pcCommand = NULL;

    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000271); //* MSG_LOG_ID_C_00000271
    }

    iCntLangType = SPS_LANG_TYPE_CPP;
    if (ptParam->iLangType == LANG_TYPE_C) {
        iCntLangType = SPS_LANG_TYPE_C;
    }
    iFuncRet = _itot_s(iCntLangType, langTypeBuf, sizeof(langTypeBuf) / sizeof(_TCHAR), 10);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** R}h쐬 */
    CDMtrLibUtilFreeW((void **)&pcCommand);
    iFuncRet = CDMtrLibUtilCreateStringW(&pcCommand,
        pcStructSmpPath,
        ptParam->pcOptionDebug2,
        _T(" --lang_name \""), ptParam->pcLangName,
        _T("\" --lang_type \""), &langTypeBuf,
        _T("\" --file_path \""), ptParam->pcDirPath,
        _T("\" --file_name \""), ptParam->pcFName,
        _T("\" --metric_file \""), ptParam->pcResultFile,
        _T("\" --metric_skip \""), (iSkipChangeFlg == 0) ? _T("old") : _T("new"),
        _T("\" --error_output \""), ptParam->pcErrOutFile,
        _T("\" --func_opt \""), pcFuncOpt,
        _T("\" --qrt_flg \""), (giQrtOutFlg == CDMTR_TRUE) ? _T("EXCEL") : _T("NONE"), _T("\""),
        _T(" \""),
        ptParam->pcNkfFile,
        _T("\""),
        NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** 쐬R}hs */
    iFuncRet = CDMtrCntCallCreateProcess(STRUCT_SMP, pcCommand, NULL, NULL, NULL);
    if (iDebug == DEBUG_ON || iSpsDebug == DEBUG_ON) {
        CDMtrCntLogLogOutErrExec(ptParam->pcErrOutFile);
    }
    if (iFuncRet != RET_OK) {
        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000157, pcStructSmpPath); //* MSG_LOG_ID_C_00000157
    }
    else {
        iNumOfSuccess++;
    }

    iRet = RET_OK;
FUNC_END:
    CDMtrLibUtilFreeW((void **)&pcCommand);
    return iRet;
}

/*!
@brief          \gNX𑪒肷(C/C++(W))
*/
static int cdmtrCntStructureCodeMetricsStandard(_TCHAR *pcFileName, _DoStruct *ptParam)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    int iExitCode = RET_OK;
    _TCHAR *pcCommand = NULL;
    _TCHAR *pcWorkPath = NULL;
    _TCHAR *pcIncDir = NULL;

    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000272); //* MSG_LOG_ID_C_00000272
    }
    CDMtrLibUtilFreeW((void **)&pcCommand);

    /** ΏۃpX̃fBNgpX擾 */
    iFuncRet = CDMtrLibUtilMallocW((void **)&pcWorkPath, (_tcslen(pcFileName) + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibFileGetDirName(pcFileName, pcWorkPath);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** \gNXs */
    iFuncRet = CDMtrLibFileGetShortPathWithinMaxpathW(pcWorkPath, &pcIncDir);
    if (iFuncRet != RET_OK) {
        pcIncDir = ptParam->pcDirPath;
        ptParam->pcDirPath = NULL;
    }

    /** R}h쐬 */
    iFuncRet = CDMtrLibUtilCreateStringW(&pcCommand,
        pcStructPath,
        OPTION_SEP_SPACE,
        gpcCompOpt,
        OPTION_SEP_SPACE,
        ptParam->pcLangOpt,
        _T(" -n -X \""),
        pcTempFolderName, FOLDER_SEPALATE, _T("test.xref"),
        _T("\" --error_output \""), ptParam->pcErrOutFile,
        _T("\" --metric_file \""), ptParam->pcResultFile,
        _T("\" --target_file_path \""), ptParam->pcFileNameLogOut,
        _T("\" --pcXrefFPath \""), pcTempXref,
        _T("\" -I\""),
        pcIncDir,
        _T("\" "),
        pcKnownOpts,
        OPTION_SEP_SPACE,
        gpcSysInc,
        OPTION_SEP_SPACE,
        _T(" --metrics_skip "), pcSkip,
        _T(" --metrics_func_opt "), pcFuncOpt,
        _T(" \""),
        ptParam->pcNkfFile,
        _T("\""),
        NULL);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }

    /** 쐬R}hs */
    iFuncRet = CDMtrCntCallCreateProcess(STRUCT, pcCommand, NULL, NULL, &iExitCode);
    if (iFuncRet != RET_OK) {
        if (iDebug == DEBUG_ON) {
            CDMtrCntLogLogOutErrExec(ptParam->pcErrOutFile);
        }
        if (iExitCode == RET_FAIL_XREF) {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000158, pcStructPath); //* MSG_LOG_ID_C_00000158
            iNumOfSuccess++;
        }
        else {
            CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000159, pcStructPath); //* MSG_LOG_ID_C_00000159
        }
    }
    else {
        iNumOfSuccess++;
    }

    iRet = RET_OK;
FUNC_END:
    CDMtrLibUtilFreeW((void **)&pcCommand);
    CDMtrLibUtilFreeW((void **)&pcWorkPath);
    CDMtrLibUtilFreeW((void **)&pcIncDir);
    return iRet;
}

/** Function : cdmtrCntStructureDumpAllListsIntoXML
 *  -----------------------------------------------
 *  Parse the highest-level list i.e '_DoStructLists'
 *  instance 'doStructLists' and dumps the data for 
 *  each non-null field into an xml file.
 *
 *  @return iRet           : Success or failure.
 */
static int cdmtrCntStructureDumpAllListsIntoXML(){


    /** Check for the _DoStructLists instance. */
    if ( doStructLists == NULL ){
        return RET_NG;
    }
    if ( doStructLists->javaList != NULL ){ /** Process java list. */

        cdmtrCntStructureDumpListIntoXML( doStructLists->javaList->listHead, LANG_TYPE_JAVA );
    }
    if ( doStructLists->csList != NULL ){ /** Process cs list. */

        cdmtrCntStructureDumpListIntoXML( doStructLists->csList->listHead, LANG_TYPE_CS );
    }
    if ( doStructLists->c_cppList!= NULL ){ /** Process c_cpplist. */

        cdmtrCntStructureDumpListIntoXML( doStructLists->c_cppList->listHead, LANG_TYPE_CPP);
    }
    if ( doStructLists->jsList != NULL){ /** Process js_List. */

        cdmtrCntStructureDumpListIntoXML( doStructLists->jsList->listHead, LANG_TYPE_JS );
    }
    if ( doStructLists->vbList != NULL){ /** Process vb List. */

        cdmtrCntStructureDumpListIntoXML( doStructLists->vbList->listHead, LANG_TYPE_VB );
    }

    return RET_OK;
}

/** Function : cdmtrCntStructureDumpListIntoXML
 *  -----------------------------------------------
 *  Parses the chain of _DoStructNode instances and
 *  extracts the _DoStruct info from each and finally
 *  dumps this info into an xml file.
 *
 *  @param  doStructNode   : First _DoStructNode instance
 *                           in the chain(list).
 *
 *  @param  langType       : Language type corresponding 
 *                           to which _DoStruct info is
 *                           being dumped.
 *
 *  @return iRet           : Success or failure.
 */
static int cdmtrCntStructureDumpListIntoXML( _DoStructNode *doStructNode, int langType ){


    FILE *outputXMLFile = NULL;                       /** Pointer for the file into which xml will be dumped. */

    int iRet = RET_NG;
    int allocSize = 0;
    char *filePath = NULL;                            /** Full path of the directory containing the xml file. */
    char *xmlFileName = NULL;                            /** Name of file into which xml will be dumped,this will be specific to each language. */
    char *xmlFilePath = NULL;
    _DoStructNode *currDoStructNode = NULL;

    /** Declaration of various node pointers that will be used comments on the right show the corresponding '_DoStruct' fields. */

    mxml_node_t *xml = NULL;                  /** Parent xml node. */
    mxml_node_t *file = NULL;                 /** Node for each fileDoStruct. */
    mxml_node_t *LangOpt = NULL;              /** Node for pcLangOpt. */
    mxml_node_t *charset = NULL;              /** Node for pcOptionCharset. */
    mxml_node_t *dirPath = NULL;              /** Node for pcDirPath. */
    mxml_node_t *inputFileName = NULL;        /** Node for pcFName. */
    mxml_node_t *inputFilePath = NULL;        /** Node for pcFileNameLogout. */
    mxml_node_t *langTag = NULL;              /** Node for javaTag. */

    /** Variables to hold the values inside of various tags. */
    char *langOptValue = NULL;
    char *charsetValue = NULL;
    char *dirPathValue = NULL;
    char *inputFileNameValue = NULL;
    char *inputFilePathValue = NULL;
    char *token = NULL;

    _DoStruct *tempDoStruct = NULL;

    if ( doStructNode == NULL ){
        CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000728, __FILE__, __LINE__ );     //!<MSG_LOG_ID_C_00000728
        return iRet;
    }
    currDoStructNode = doStructNode;   /** Temp parser to parse each '_DoStructNode' element. */

    /** Assign absolute path of the folder into which the xml will be dumped into filePath. */
    iRet = CDMtrLibUtilUNICODEtoMBCS( &filePath, &pcTempFolderName, pcResultCodeOpts );
    if ( iRet != RET_OK ){
        CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000727, __FILE__ , __LINE__ );
        return iRet;
    }		

    /** Allocating the node for the parent xml tag. */
    xml = mxmlNewXML( "1.0" );

    /** Allocating node for the language specific tag. */
    langTag = mxmlNewElement( xml, "lang" );

    /** Obtain fileName according to the lang type. */
    xmlFileName = cdmtrCntStructureGetXMLFileName( langType );
    if ( !strcmp( xmlFileName, "errorFile" ) ){
        CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000727, __FILE__, __LINE__ );
        return RET_NG;
    }

    allocSize = strlen(filePath) + XREF_SEPARATOR + strlen(xmlFileName) + NULL_TERM_LENGTH;
    xmlFilePath = (char *)CDMtrLibCmnMalloc(allocSize);
    if ( xmlFilePath == NULL )
        return RET_NG;
    strcpy( xmlFilePath, filePath );
    strcat( xmlFilePath, FOLDER_SEPALATE_CHAR_TYPE );
    strcat( xmlFilePath, xmlFileName );


    /** Parse throug the entire langList and prepare necessary tags for each '_DoStruct' instance.*/
    while ( currDoStructNode != NULL ){

        langOptValue = NULL;
        charsetValue = NULL;
        //dirPathValue = NULL;
        //inputFileNameValue = NULL;
        inputFilePathValue = NULL;
        token = NULL;


        /** Obtain the DoStruct element. */
        tempDoStruct = & ( currDoStructNode->doStruct );

        if ( tempDoStruct -> pcLangOpt != NULL ){
            iRet = CDMtrLibUtilUNICODEtoMBCS( &langOptValue, & ( tempDoStruct -> pcLangOpt ), ENCODING_UTF_8 );
            if ( iRet != RET_OK ){
                CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000733, __FILE__, __LINE__ );
                goto WHILE_END;
            }
        }
        else{
            langOptValue = "null value";
        }
        iRet = CDMtrLibUtilUNICODEtoMBCS( &token, & ( tempDoStruct -> pcOptionCharset ), ENCODING_UTF_8 );
        if( ( charsetValue = strtok( token, SPACE ) ) == NULL )
            goto WHILE_END;
        if( ( charsetValue = strtok( NULL, SPACE ) ) == NULL )
            goto WHILE_END;
        if ( iRet != RET_OK ){
            CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000733, __FILE__, __LINE__ );
            goto WHILE_END;
        }
        iRet = CDMtrLibUtilUNICODEtoMBCS( &dirPathValue, & ( tempDoStruct -> pcDirPath ), ENCODING_UTF_8 );
        if ( iRet != RET_OK ){
            CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000733, __FILE__, __LINE__ );
            goto WHILE_END;
        }
        iRet = CDMtrLibUtilUNICODEtoMBCS( &inputFileNameValue, & ( tempDoStruct -> pcFName ), ENCODING_UTF_8 );
        if ( iRet != RET_OK ){
            CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000733, __FILE__, __LINE__ );
            goto WHILE_END;
        }
        iRet = CDMtrLibUtilUNICODEtoMBCS( &inputFilePathValue, & ( tempDoStruct -> pcFileNameLogOut ) , ENCODING_UTF_8 );
        if ( iRet != RET_OK ){
            CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000733, __FILE__, __LINE__ );
            goto WHILE_END;
        }

        /** Allocating the node for the file tag. */
        file = mxmlNewElement( langTag, "file" );                                          

        /** Allocating the node for the LangOpt tag and assigning value. */
        LangOpt = mxmlNewElement( file, "langOpt" ); 
        mxmlNewText( LangOpt, 0, langOptValue );

        /** Allocating the node for the charset tag and assigning value. */
        charset = mxmlNewElement( file, "charset" );		
        mxmlNewText( charset, 0, ENCODING_UTF_8_CHAR_PTR );

        /** Allocating the node for the dirpath tag and assigning value. */
        dirPath = mxmlNewElement( file, "dirPath" );		
        mxmlNewText( dirPath, 0, dirPathValue );		

        /** Allocating the node for the filename tag and assigning value. */
        inputFileName = mxmlNewElement( file, "fileName" );
        mxmlNewText( inputFileName, 0, inputFileNameValue );		

        /** Allocating the node for the inputFile tag and assigning value. */
        inputFilePath = mxmlNewElement( file, "inputFilePath" );	
        mxmlNewText( inputFilePath, 0, inputFilePathValue );

WHILE_END:	
        currDoStructNode = currDoStructNode->next;
        CDMtrLibUtilFreeW((void **)& ( token ));
        CDMtrLibUtilFreeW((void **)& ( dirPathValue ));
        CDMtrLibUtilFreeW((void **)& ( inputFileNameValue ));
        CDMtrLibUtilFreeW((void **)& ( inputFilePathValue ));
    }

    /** Open xml file into which data will be dumped. */
    outputXMLFile = fopen( xmlFilePath, "w" );
    if ( outputXMLFile == NULL ){
        CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000727 );
        return RET_NG;
    }

    /** Dump data into the file. */
    iRet = mxmlSaveFile( xml, outputXMLFile, MXML_NO_CALLBACK );
    if ( iRet == RET_NG ){
        CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000727 );
        return RET_NG;
    }

    fclose( outputXMLFile );
    mxmlRelease( xml );
    CDMtrLibUtilFreeW((void **)& ( filePath ));
    CDMtrLibUtilFreeW((void **)& ( xmlFilePath ));

    return RET_OK;
}



/** Function : cdmtrCntStructureInitializeDoStructLists
 *  ---------------------------------------------------
 *  Initialize a global instance of _DoStructLists.
 *
 *  @return iRet       : Success or failure.
 */
static int cdmtrCntStructureInitializeDoStructLists(){

    doStructLists = ( _DoStructLists *) CDMtrLibCmnMalloc ( sizeof( _DoStructLists ) );
    if ( doStructLists == NULL ){
        CDMtrLibCmnLogOut( MSG_LOG_ID_C_00000732, __FILE__, __LINE__ );
        return RET_NG;
    }
    return RET_OK;
}

/** Function : cdmtrCntStructureFetchAndUpdateINumOfSuccessJava
 *  -----------------------------------------------------------
 *  Obtain the number of successfully processed files by the 
 *  java parser module and set the corresponding global 
 *  variable(iNumOfSuccessJava) accordingly.
 *
 *  @return iRet       : Success or failure.
 */
static int cdmtrCntStructureFetchAndUpdateINumOfSuccessJava()
{
    int iRet = RET_NG;
    static FILE *ptrToReadINumSuccessInfo = NULL;
    _TCHAR *parserOutFilePath = NULL;
    _TCHAR *buff = NULL;
    _TCHAR *delim = COMMA;
    _TCHAR *token = NULL;
    
    if( ( buff = (_TCHAR *)CDMtrLibCmnMalloc( sizeof(char) * maxStrSize )) == NULL ){
        iNumOfSuccessJava = 0;
        goto FUNC_END;
    }
    /** Allocate memory for dataStr. */
    CDMtrLibFileSetDirFileName( pcTempFolderName, TEMP_JPARSER_OUTFILE, &parserOutFilePath );       

    /** Open file created by ncmjparser for reading. */
    ptrToReadINumSuccessInfo=_tfopen( parserOutFilePath, _T("r+") );	
    if ( ptrToReadINumSuccessInfo == NULL ){
        CDMtrLibCmnErrLogOut( MSG_LOG_ID_C_00000729 ); //* MSG_LOG_ID_C_00000729
        goto FUNC_END;
    }
    else
    {		   
        while( !feof(ptrToReadINumSuccessInfo) )
        {  
            memset( buff, 0x00, maxStrSize ); 
            /** Read file  using fscanf. */
            fwscanf( ptrToReadINumSuccessInfo, _T("%[^\n]\n"), buff );  
        }

        /** Obtain the value of iNumOfSuccess from the buffer. */
        if((token = _tcstok( buff, delim )) == NULL )
            goto FUNC_END;
        while( token != NULL ){
            /** Keep reading from the buffer. */
            if((token = _tcstok( NULL,delim )) == NULL )
                goto FUNC_END;
            /** If the string recently read is "JAVA", next value will be that of iNumofSuccess. */
            if( wcscmp( token, UNC_LANGOPT_JAVA ) == 0 ){
                /** Get the Value of iNumOfSuccess. */		    
                if((token = _tcstok( NULL, delim )) == NULL )
                    goto FUNC_END;
                token++;
                break;
            }
        }
        swscanf( token, _T("%d"), &iNumOfSuccessJava );
        fclose( ptrToReadINumSuccessInfo );			 
    }
    iRet = RET_OK;
FUNC_END:
    if( buff != NULL ){
        CDMtrLibUtilFreeW((void **)&buff);
    }
    if( parserOutFilePath != NULL ){
        CDMtrLibUtilFreeW((void **)&parserOutFilePath);
    }
    return iRet;
}


/** Function : cdmtrCntStructureGetXMLFileName
 *  -----------------------------------------------
 *  Based on the parameter, returns the name of the
 *  intermediate xml file into which _DoStruct info
 *  corresponding to a specific language will be dumped.
 *
 *  @param  langType   : Integer denoting language
 *                       type for which _DoStruct info
 *                       will be dumped.
 *
 *  @return xmlFileName: Name of the xml file.
 */
static char* cdmtrCntStructureGetXMLFileName(int langType){


    char *xmlFileName =	NULL;

    switch ( langType ){

    case LANG_TYPE_JAVA:
        xmlFileName = "javaData.xml";
        break;
    case LANG_TYPE_CS:
        xmlFileName = "cs_data.xml";
        break;
    case LANG_TYPE_C:
    case LANG_TYPE_CPP:
        xmlFileName = "c_cpp_data.xml";
        break;
    case LANG_TYPE_VB:
        xmlFileName = "vb_data.xml";
        break;
    case LANG_TYPE_JS:
        xmlFileName = "js_data.xml";
        break;
    default:
        xmlFileName = "errorFile";
        break;
    }

    return xmlFileName;
}

/** Function : cdmtrCntStructureReleaseDoStruct
 *  -------------------------------------------
 *  Deallocates the _DoStruct instance.
 *
 *  @param  doStruct   : Instance to be
 *                       deallocated.
 */
static void cdmtrCntStructureReleaseDoStruct( _DoStruct doStruct ){

    CDMtrLibUtilFreeW((void **)& ( doStruct.pcNkfFile ));
    CDMtrLibUtilFreeW((void **)& ( doStruct.pcResultFile ));
    CDMtrLibUtilFreeW((void **)& ( doStruct.pcOptionCharset ));
    CDMtrLibUtilFreeW((void **)& ( doStruct.pcDirPath ));
    CDMtrLibUtilFreeW((void **)& ( doStruct.pcFName ));
    CDMtrLibUtilFreeW((void **)& ( doStruct.pcLangName ));
    CDMtrLibUtilFreeW((void **)& ( doStruct.pcErrOutFile ));
}

/** Function : cdmtrCntStructureReleaseDoStructNode
 *  -----------------------------------------------
 *  Deallocates the _DoStructNode instances.
 *
 *  @param  doStructNode   : Instance to be
 *                           deallocated.
 */
static void cdmtrCntStructureReleaseDoStructNode( _DoStructNode *doStructNode ){

    /** Temp variable to hold doStructNode value. */
    _DoStructNode *theCurrTempDoStructNode = NULL;
    while ( doStructNode != NULL ){
        /** Delete the _DoStruct instance.*/
        cdmtrCntStructureReleaseDoStruct( doStructNode->doStruct );
        /** Load the current _DoStructNode's address. */
        theCurrTempDoStructNode = doStructNode;
        /** Move the head to the next _DoStructNode instance. */
        doStructNode = doStructNode->next;
        /** Release the current _DoStructNode. */
        CDMtrLibUtilFreeW((void **)& ( theCurrTempDoStructNode ));		
    }
}

/** Function : cdmtrCntStructureReleaseDoStructLists
 *  ------------------------------------------------
 *  Deallocates the non-null fields of doStructLists
 *  instance.
 */
static void cdmtrCntStructureReleaseDoStructLists(){

    if( doStructLists == NULL){
        return;
    }
    if ( doStructLists->javaList != NULL ){
        cdmtrCntStructureReleaseDoStructNode( doStructLists->javaList->listHead );
    }
    if ( doStructLists->jsList != NULL ){
        cdmtrCntStructureReleaseDoStructNode( doStructLists->jsList->listHead );
    }
    if ( doStructLists->csList != NULL ){
        cdmtrCntStructureReleaseDoStructNode( doStructLists->csList->listHead );
    }
    if ( doStructLists->c_cppList != NULL ){
        cdmtrCntStructureReleaseDoStructNode( doStructLists->c_cppList->listHead );
    }
    if ( doStructLists->vbList != NULL ){
        cdmtrCntStructureReleaseDoStructNode( doStructLists->vbList->listHead );
    }	
    CDMtrLibUtilFreeW((void **)& ( doStructLists->javaList ));
    CDMtrLibUtilFreeW((void **)& ( doStructLists->jsList ));
    CDMtrLibUtilFreeW((void **)& ( doStructLists->csList ));
    CDMtrLibUtilFreeW((void **)& ( doStructLists->c_cppList ));
    CDMtrLibUtilFreeW((void **)& ( doStructLists->vbList ));
    CDMtrLibUtilFreeW((void **)& ( doStructLists ));
}

/** Function : cdmtrCntStructureInvokeAndProcess
 *  -----------------------------------------------
 *  Invokes the java parser module and obtains the 
 *  total number of successfully processed files
 *  from the parser output.
 *
 *  @param  langType   : Integer denoting language
 *                       type for which parser will
 *                       be invoked.
 *
 *  @return iRet       : Success or failure.
 */
static int  cdmtrCntStructureInvokeAndProcess( int langType ){

    int iFuncRet = RET_NG ;

    switch( langType ){

    case LANG_TYPE_JAVA:

        /** Invoke the JAVA specific parser 'ncmjparser.jar'. */
        iFuncRet = cdmtrCntStructureCodeMetricsJAVA();
        if ( iFuncRet != RET_OK ) {
            CDMtrLibCmnErrLogOut( MSG_LOG_ID_C_00000734 ); 
            return RET_NG;
        }
        /** Fetch and update the line containing information for iNumOfSuccessJava */
        iFuncRet = cdmtrCntStructureFetchAndUpdateINumOfSuccessJava();
        if ( iFuncRet != RET_OK ) {
            CDMtrLibCmnErrLogOut( MSG_LOG_ID_C_00000726 ); 
            return RET_NG;
        }
        break;
    default:
        break;
    }
    return RET_OK;
}


/** Function : cdmtrCntStructureAppendParserOutput
 *  -----------------------------------------------
 *  Appends the output of individual non-java 
 *  parsers into a single file.
 *
 *  @param  resultFile   : File containing parser
 *                         output.
 *
 *  @return iRet         : Success or failure.
 */
static int cdmtrCntStructureAppendParserOutput( _TCHAR *resultFile ){

    FILE *parserOutput = NULL;
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    _TCHAR *dataBuffer = NULL;
    _TCHAR *pcRet = NULL;
    char *dataOutBuffer = NULL;
    int bomOffset = 0;
    long lSize = 0;
    size_t sizeOfLineRead = 0;

    /** Open the parser output. */
    if (( parserOutput = _tfopen( resultFile, _T("r, ccs=UTF-8"))) == NULL ) {
        CDMtrLibCmnTrcLogOut( MSG_LOG_ID_C_00000114, resultFile ); //* MSG_LOG_ID_C_00000114
        goto FUNC_END;
    }

    bomOffset = ftell( parserOutput );
    if ( fseek( parserOutput, 0L, SEEK_END ) != 0 ) {
        goto FUNC_END;
    }
    lSize = ftell( parserOutput );
    if ( lSize <= 0 ) {
        goto FUNC_END;
    }
    lSize++;
    if ( fseek( parserOutput, bomOffset, SEEK_SET ) != 0 ) {
        goto FUNC_END;
    }
    lSize += 6 * 8;
    iFuncRet = CDMtrLibUtilMallocW((void **)&dataBuffer, (lSize + 1) * sizeof(_TCHAR) );
    if( iFuncRet != RET_OK ){
        goto FUNC_END;
    }

    /** Read file line by line and store into the buffer. */
    while (1) {
        pcRet = _fgetts( dataBuffer, (int)lSize, parserOutput );
        /** Find the length of the line read and update the size
          * of maxStrSize if it is smaller.
          */
        sizeOfLineRead = wcslen(dataBuffer);
        if( maxStrSize < sizeOfLineRead ){
            maxStrSize = sizeOfLineRead;
        }
        if(pcRet != NULL){
            iFuncRet = CDMtrLibUtilUNICODEtoMBCSW( &dataOutBuffer, &dataBuffer, ENCODING_UTF_8 );
            if ( iFuncRet != RET_OK ) {
                goto FUNC_END;
            }
            fputs( dataOutBuffer, spcOutFile );
             CDMtrLibUtilFreeW( (void **)&dataOutBuffer );
        }
        else
            break;
    }
    iRet = RET_OK;
FUNC_END:
    /** Append this buffer into spcoutfile. */
    if ( dataBuffer != NULL ) {					
        CDMtrLibUtilFreeW( (void **)&dataBuffer );
    }
    /** Close the parser output. */
    if( parserOutput != NULL ){
        fclose( parserOutput );
        parserOutput = NULL ;
    }
    return iRet;
}

/** Function : cdmtrCntStructureGetAbsFilePath
 *  -----------------------------------------------
 *  Parses the first argument and obtains the absolute
 *  path of the file using the first two comma
 *  separated fields.
 *
 *  @param  line   : Comma separated string.
 *                      
 *  @param charRet : Absolute file path obtained
 *                    from first two fields.
 *
 *  @return iRet   : Success or failure.
 */
static int cdmtrCntStructureGetAbsFilePath( char *line, char **charRet ){

    char *stringRecieved = NULL;
    char *tokenFileName = NULL;
    char *tokenDirName = NULL;
    char *delimQuotes = DOUBLEQUOTATION_CHAR_TYPE;
    int iRet = RET_NG;

    /** Allocate memory for the string to be processed. */
    if((stringRecieved = (char *)CDMtrLibCmnMalloc( strlen(line) + 1 )) == NULL){
        goto FUNC_END;
    }
    /** Safely store the string to be processed. */
    strcpy( stringRecieved, line );
    tokenDirName = strtok ( stringRecieved, delimQuotes );
    if( tokenDirName == NULL )
        goto FUNC_END;
    strtok( NULL, delimQuotes );
    tokenFileName = strtok( NULL, delimQuotes );
    if( tokenFileName == NULL )
        goto FUNC_END;
    /** Assign memory for string to return. */
    if((*charRet = (char *)CDMtrLibCmnMalloc( strlen( tokenDirName ) + wcslen( FOLDER_SEPALATE ) + strlen( tokenFileName ) + 1)) == NULL){
        goto FUNC_END;
    }
    strcpy( *charRet, tokenDirName );
    /** Concat the file name to the directory name. */
    strcat( *charRet, FOLDER_SEPALATE_CHAR_TYPE );
    strcat( *charRet, tokenFileName );
    iRet = RET_OK;
FUNC_END:
    CDMtrLibUtilFreeW( (void **)&stringRecieved );
    return iRet;
}


/** Function : cdmtrCntStructureReadDataFromFile
 *  -----------------------------------------------
 *  Reads a line from the given file and stores
 *  the line in a char pointer.
 *
 *  @param  fileToRead : File from which line will
 *                       be read.
 *
 *  @return dataString : Line read from file.
 */
static char*  cdmtrCntStructureReadDataFromFile(FILE *fileToRead)
{

    int filePointerPos = SEEK_SET;
    char *dataString = NULL;
    
    /** Allocate memory for the line to be read. */
    dataString = (char *)CDMtrLibCmnMalloc( (maxStrSize) * sizeof(char));
    /** Store file pointer position. */
    filePointerPos = ftell(fileToRead);
    while(1)
    {
        /** Read a line from file. */
        if((fgets( dataString, maxStrSize, fileToRead )) == NULL)
        {
            /** EOF reached. */            
            CDMtrLibUtilFreeW( (void **)&dataString );
            dataString = NULL;
            break;
        }
        /** Check if last character is indeed a newline. */
        if( dataString[strlen(dataString)-1] != '\n'){
            /** Memory too small, needs to be reallocated. */
            CDMtrLibUtilFreeW( (void **)&dataString );
            /** Increase max string length. */
            maxStrSize*=2;
            if( (dataString = (char *)CDMtrLibCmnMalloc( (maxStrSize) * sizeof(char))) == NULL )
                return NULL;
            fseek(fileToRead, filePointerPos, SEEK_SET);            
        }
        else
            break;       
    }    
    return dataString;
}


/**
 * Function : cdmtrCntStructurePrintErrToOutFile
 * -------------------------------------------------------------
 * Prints data for a file that was not processed
 * on the parser module end. 
 *
 * @param  currOpen    : Pointer to the file into which file
 *                       information is supposed to be dumped.
 *
 * @param  currOpenName: Name of the currently opened file.
 *
 * @param  fileInList  : Name of the input file which could not
 *                       be processed on the parser module end.
 *
 * @return int  : Success or failure.
 */
static int   cdmtrCntStructurePrintErrToOutFile(FILE *currOpen, _TCHAR *currOpenName, _TCHAR *fileInList){

    _TCHAR *fileAbsPath = NULL;
    FILE *storeOutputFile = NULL;
    /** Adjust file pointers so that spcOutFile points to newNcmeter. */
    /** Close the newNcmeter file pointer. */
    fclose(currOpen);
    currOpen = NULL;    
    if(spcOutFile != NULL){
        storeOutputFile = spcOutFile;
        spcOutFile = NULL;
    }
    /** Open newNcmeter file with scpOutFile file pointer. */
    if( (spcOutFile = _tfopen(currOpenName, _T("a+"))) == NULL ) {
        CDMtrLibCmnErrLogOut(MSG_LOG_ID_C_00000114, currOpenName); //* MSG_LOG_ID_C_00000114
        return RET_NG;
    }
    /** Get proper file path from tempStr. */
    fileAbsPath = (_TCHAR *)CDMtrLibFileRemoveLongPathMarkW(fileInList);
    /** Call the function to print error information. */
    CDMtrCntOutputCSVErrFileToOutFile( fileAbsPath, UNC_LANGOPT_JAVA, TRUE );
    /** Again adjust the file pointers for further editing. */
    if( spcOutFile != NULL ){
        fclose(spcOutFile);
        spcOutFile = NULL;
    }
    spcOutFile = storeOutputFile;
    currOpen = _tfopen( currOpenName, _T("a"));
    if (currOpen == NULL){
        CDMtrLibCmnErrLogOut( MSG_LOG_ID_C_00000114, currOpenName); //* MSG_LOG_ID_C_00000114
        return RET_NG;
    }
    else{
        fseek( currOpen, 0, SEEK_END);
    }
    return RET_OK;
}